//+------------------------------------------------------------------+
//|                                                 RSIstudy_V1a.mq4 |
//|                                         Copyright 2018, Iwori_Fx |
//|                           https://www.mql5.com/en/users/iwori_fx |
//+------------------------------------------------------------------+
#property copyright "Copyright 2018, Iwori_Fx"
#property link      "https://www.mql5.com/en/users/iwori_fx"
#property version   "1.00"
#property strict
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_plots   2


//--- plot High
#property indicator_label1  "rising"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrGreen
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2
//--- plot Low
#property indicator_label2  "falling"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrMaroon
#property indicator_style2  STYLE_SOLID
#property indicator_width2  2

#property indicator_level1 0.0
#property indicator_level2 50.0
#property indicator_level3 100.0
#property indicator_level4 25.0
#property indicator_level5 75.0

//input ENUM_APPLIED_PRICE   src = PRICE_CLOSE;   // Source
input int                  window01 = 10;       // RSI Length
input int                  window02 = 5;        // RSI Smoothing
input double               e = 2.71;            // Inverse Fisher E Value




//--- indicator buffers
double         HighBuffer[];
double         LowBuffer[];
double         fisherBuffer[];
double         rsiBuffer[];
double         srcBuffer[];

int lastTF = Period();

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
//--- indicator buffers mapping
   IndicatorBuffers(5);
   
   SetIndexBuffer(0,HighBuffer);
   SetIndexBuffer(1,LowBuffer);
   SetIndexBuffer(2,fisherBuffer);
   SetIndexBuffer(3,rsiBuffer);
   SetIndexBuffer(4,srcBuffer);
   
   SetIndexStyle(2,DRAW_NONE);
   SetIndexStyle(3,DRAW_NONE);
   SetIndexStyle(4,DRAW_NONE);
   
   IndicatorShortName("RSI Inverse Fisher Transform V1c");
   
//---
   return(INIT_SUCCEEDED);
}
  
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
//---
   
   
   int limit, calculated;
   double rsi;
   
   calculated = prev_calculated;
   if(rates_total<2) return 0;
   if(calculated<0) return 0;
   
   if(lastTF!=Period())
   {
      lastTF = Period();
      calculated = 0;
   }
   
   limit=rates_total-calculated;
   if(calculated>0)limit++;
   
   for(int i=limit-1; i>=0; i--)
   {  
      fisherBuffer[i] = srcBuffer[i] = 0.0;
      HighBuffer[i] = LowBuffer[i] = EMPTY_VALUE;
      if(i>=rates_total-1) continue;
      
      srcBuffer[i] = (High[i]+Low[i]+Open[i]+Close[i])/4.0;
      rsiBuffer[i] = iRSIOnArray(srcBuffer,0,window01,i);
      rsi = (iMAOnArray(rsiBuffer,0,window02,0,MODE_EMA,i) - 50.0) * 0.1;
      
      fisherBuffer[i] = (MathPow(e,2.0*rsi) - 1.0) / (MathPow(e,2.0*rsi) + 1.0)*50.0 + 50.0;
      
      if(fisherBuffer[i]>fisherBuffer[i+1])
      {
         HighBuffer[i] = fisherBuffer[i];
         HighBuffer[i+1] = fisherBuffer[i+1];
      }
      else if(fisherBuffer[i]<fisherBuffer[i+1]) 
      {
         LowBuffer[i] = fisherBuffer[i];
         LowBuffer[i+1] = fisherBuffer[i+1];
      }
      else
      {
         HighBuffer[i] = HighBuffer[i+1];
         LowBuffer[i] = LowBuffer[i+1];
      }
   }
//--- return value of prev_calculated for next call
   return(rates_total);
}
//+------------------------------------------------------------------+
